/***************************************************************************
 *   Copyright (C) 2007 by Dr. Feynman                                     *
 *   dr@feynman.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef WIN32

#include <winsock2.h>
#include <sys/timeb.h>
#include "feynman_win32.h"

#define MAX_SOCKETS       512
#define NUM_SOCK_FDSETS  ((MAX_SOCKETS+sizeof(fd_set)-1) / sizeof(fd_set))

int poll(struct pollfd *p, int num, int timeout)
{
    struct timeval tv;
    FD_SET read  [NUM_SOCK_FDSETS];
    FD_SET write [NUM_SOCK_FDSETS];
    FD_SET except[NUM_SOCK_FDSETS];
    int i, n, ret;

    memset(read, 0, sizeof(read));
    memset(write, 0, sizeof(write));
    memset(except, 0, sizeof(except));


    n = -1;
    for(i=0; i<num; i++)
    {
        if(p[i].fd < 0) continue;
        if(p[i].events & POLLIN) FD_SET(p[i].fd, &read[0]);
        if(p[i].events & POLLOUT) FD_SET (p[i].fd, &write[0]);
        if(p[i].events & POLLERR) FD_SET (p[i].fd, &except[0]);
        if(p[i].fd > n) n = p[i].fd;
    }

    if(n == -1)
    {
        ret = 0;
    }
    else if(timeout < 0)
    {
        ret = select (n+1, read, write, except, NULL);
    }
    else
    {
        tv.tv_sec  = timeout / 1000;
        tv.tv_usec = 1000 * (timeout % 1000);
        ret = select (n+1, read, write, except, &tv);
    }

    for(i=0; ret>=0 && i<num; i++)
    {
        p[i].revents = 0;
        if(FD_ISSET (p[i].fd, &read[0])) p[i].revents |= POLLIN;
        if(FD_ISSET (p[i].fd, &write[0])) p[i].revents |= POLLOUT;
        if(FD_ISSET (p[i].fd, &except[0])) p[i].revents |= POLLERR;
    }

    return ret;
}
/*
void gettimeofday(struct timeval *tv,void* time_zone)
{
    struct _timeb cur;
    _ftime(&cur);
    tv->tv_sec = cur.time;
    tv->tv_usec = cur.millitm * 1000;
}
*/
#endif
