using System;
using System.Runtime.InteropServices;
using Feynman;

public struct t_feynman_ecm
{
    public ushort caid;
    public int provid;
    public ushort sid;
    public IntPtr ecm;
    public int ecm_len;
    public int decrypt_status;
    public IntPtr dcw;
    public int dcw_len;
    public ushort session;
    public IntPtr client;
    public int card_id;
//             struct timeval time_added;
    public IntPtr next;
}

public struct t_feynman_emm
{
    public ushort caid;
    public int provid;
    public int sa;
    public int ua;
    public IntPtr emm;
    public int emm_len;
    public int card_id;
    public int emm_id;
//             struct timeval time_added;
    public IntPtr next;
}

namespace TestFeynman
{
    class TestFeynmanServer
    {
        [DllImport("libfeynman")]
        public static unsafe extern void feynman_set_ecm_dcw(ref t_feynman_ecm ecm, byte[] dcw, int len);

        private static int processECM(int notUsed, ref t_feynman_ecm ecm)
        {
            Console.WriteLine("[ Test server ] Sending ecm to card");

            byte[] ecm_data = new byte[ecm.ecm_len];
            byte[] dcw = new byte[0x10];

            Marshal.Copy(ecm.ecm, ecm_data, 0, ecm.ecm_len);

            Console.Write("[ Test server ] ECM: ");
            for(int i=0; i<ecm.ecm_len; i++)
            {
                Console.Write("{0:x} ", ecm_data[i]);
            }
            Console.Write("\n");

            for(int i=0; i<0x10; i++)
            {
                dcw[i] = 0x12;
            }
            feynman_set_ecm_dcw(ref ecm, dcw, 0x10);
            return 1;
        }

        private static int processEMM(int notUsed, ref t_feynman_emm emm)
        {
            Console.WriteLine("[ Test server ] Sending emm to card");

            byte[] emm_data = new byte[emm.emm_len];
            Marshal.Copy(emm.emm, emm_data, 0, emm.emm_len);

            Console.Write("[ Test server ] EMM: ");
            for(int i=0; i<emm.emm_len; i++)
            {
                Console.Write("{0:x} ", emm_data[i]);
            }
            Console.Write("\n");
            return 1;
        }

        public static void Main(string[] args)
        {
            FeynmanServer server = new FeynmanServer(12345, processECM, processEMM);

            int access_mode = 0x01 | 0x02 | 0x04 | 0x08 | 0x10;
            server.addUser("username", "password", access_mode);
            server.addCard(0x0b00, 0x00000000, 0x123456, 0x12345678);
            server.start();
            while(true)
            {
                System.Threading.Thread.Sleep(5000);
            }
        }
    }
}